package cz.zweistein.gw.predictor;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.LinkedList;
import java.util.List;
import java.util.Locale;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.protocol.HTTP;
import org.apache.http.util.EntityUtils;
import org.apache.log4j.Logger;

import cz.zweistein.gw.predictor.model.User;

public class XTHRemote {
	private static Logger logger = Logger.getLogger(XTHRemote.class);

	private static int NUMBER_OF_VOTABLE_ITEMS = 8;

	private HttpClient httpClient = new DefaultHttpClient();

	private String emailAddress = null;
	private String password = null;
	private String userId = null;

	public boolean isLoggedIn() {
		if (this.userId != null) {
			return true;
		} else {
			return false;
		}
	}

	public void logIn(User user) {

		logger.info(user.getEmail() + " logging in.");

		HttpPost httpost = new HttpPost(
				"https://www.guildwars.com/competitive/xunlai_tournament_house/login.php?lang=en");

		List<NameValuePair> nvps = new ArrayList<NameValuePair>();

		nvps.add(new BasicNameValuePair("email_address", user.getEmail()));
		nvps.add(new BasicNameValuePair("password", user.getPassword()));
		nvps.add(new BasicNameValuePair("sendForm", "sendForm"));

		try {
			httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));

			HttpResponse response = this.httpClient.execute(httpost);
			HttpEntity entity = response.getEntity();
			String pageContent = EntityUtils.toString(entity);

			if (pageContent.contains("Please try again.")) {
				logger.info("Failed to log in");
				// failure to log-in.
				this.emailAddress = null;
				this.password = null;
				this.userId = null;
			} else {
				logger.info("Logged in.");

				this.emailAddress = user.getEmail();
				this.password = user.getPassword();

				String userIdPattern = "<input type=\"hidden\" name=\"user_id\" value=\"([0-9]*)\" />";
				Pattern pattern = Pattern.compile(userIdPattern);
				Matcher matcher = pattern.matcher(pageContent);

				if (matcher.find()) {
					String userId = matcher.group(1);
					this.userId = userId;
				}

			}

		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public List<Guild> getQualifiedGuilds() {
		logger.info("Getting information about which guilds are qualified.");

		HttpPost httpost = new HttpPost(
				"https://www.guildwars.com/competitive/xunlai_tournament_house/login.php?lang=en");

		List<NameValuePair> nvps = new ArrayList<NameValuePair>();
		nvps.add(new BasicNameValuePair("continue", "View GvG Ballot"));
		nvps.add(new BasicNameValuePair("action", "insert"));
		nvps.add(new BasicNameValuePair("user_id", this.userId));
		nvps.add(new BasicNameValuePair("email_address", this.emailAddress));
		nvps.add(new BasicNameValuePair("password", this.password));

		List<Guild> result = new LinkedList<Guild>();

		try {
			httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));
			HttpResponse response = this.httpClient.execute(httpost);
			HttpEntity entity = response.getEntity();
			String pageContent = EntityUtils.toString(entity);

			String regEx = "class=\"dragableBox\" id=\"box(\\d+)\">([a-zA-Z ]+)<span style=\"font-size:.8em;\">\\(Q.P. ([0-9]*)\\)</span></div>";

			Pattern pattern = Pattern.compile(regEx);
			Matcher matcher = pattern.matcher(pageContent);

			while (matcher.find()) {
				Guild guild = new Guild();
				guild.setId(matcher.group(1));
				guild.setName(matcher.group(2).trim());
				guild.setQualifierPoints(Integer.parseInt(matcher.group(3)));
				result.add(guild);
			}

		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return result;
	}

	public int getGuildLadderRating(String guildName) {

		logger.info("Getting ladder rating for guild " + guildName);

		String fixedGuildName = guildName.replaceAll(" ", "%20");

		String url = "https://www.guildwars.com/competitive/ladder/get_ladder.php?batch=null&column=null&order=null&from=0&max=null&name="
				+ fixedGuildName + "&tag=null&lang=en";

		String ratingPattern = "<td><div id=\"opaquetext\">&nbsp;([0-9]+)</div>";

		String notFound = "None found.";

		int rating = 0;

		try {
			HttpGet httpget = new HttpGet(url);
			HttpResponse response = this.httpClient.execute(httpget);
			HttpEntity entity = response.getEntity();
			String pageContent = EntityUtils.toString(entity);

			if (pageContent.indexOf(notFound) > 0) {
				return 0;
			} else {
				Pattern pattern = Pattern.compile(ratingPattern);
				Matcher matcher = pattern.matcher(pageContent);

				if (matcher.find()) {
					rating = Integer.parseInt(matcher.group(1));
				}
			}
		} catch (ClientProtocolException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}

		return rating;
	}

	public void fillInPlayerTournamentHistory(List<Player> players) {

		Calendar calNow = new GregorianCalendar();
		Calendar calTournament = new GregorianCalendar(2008, 0, 1);

		while (calTournament.getTimeInMillis() < calNow.getTimeInMillis()) {

			String url = "http://www.guildwars.com/competitive/tournament"
					+ calTournament.get(Calendar.YEAR)
					+ "/"
					+ calTournament.getDisplayName(Calendar.MONTH,
							Calendar.LONG, Locale.US).toLowerCase()
					+ "/1v1resultselim.php";

			logger.info("Loading: " + url);

			try {

				HttpGet httpget = new HttpGet(url);
				HttpResponse response = this.httpClient.execute(httpget);

				HttpEntity entity = response.getEntity();
				String pageContent = EntityUtils.toString(entity);
				if (pageContent
						.indexOf("We're sorry, the page you requested could not be found.") > 0) {
					logger.info("Loading failed.");
					break;
				}
				String resultTable = pageContent.substring(pageContent
						.indexOf("Elimination Results"), pageContent
						.indexOf("Swiss Rounds Results"));

				String regEx = "<td>(.+)</td><td>(.+)</td>";

				Pattern pattern = Pattern.compile(regEx);
				Matcher matcher = pattern.matcher(resultTable);

				while (matcher.find()) {
					int order = Integer.parseInt(matcher.group(1));
					String playerName = matcher.group(2).trim();

					for (Player player : players) {
						if (player.getName().equals(playerName)) {
							player.addOrder(order);
							break;
						}
					}
				}

				calTournament.add(Calendar.MONTH, 1);

			} catch (ClientProtocolException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}

	}

	public void fillInGuildTournamentHistory(List<Guild> guilds) {

		Calendar calNow = new GregorianCalendar();
		Calendar calTournament = new GregorianCalendar(2008, 0, 1);

		while (calTournament.getTimeInMillis() < calNow.getTimeInMillis()) {

			String url = "http://www.guildwars.com/competitive/tournament"
					+ calTournament.get(Calendar.YEAR)
					+ "/"
					+ calTournament.getDisplayName(Calendar.MONTH,
							Calendar.LONG, Locale.US).toLowerCase()
					+ "/gvgresultselim.php";

			logger.info("Attempting to load " + url);

			try {
				HttpGet httpget = new HttpGet(url);
				HttpResponse response = this.httpClient.execute(httpget);

				HttpEntity entity = response.getEntity();
				String pageContent = EntityUtils.toString(entity);
				if (pageContent
						.indexOf("We're sorry, the page you requested could not be found.") > 0) {
					logger.info("Page load failed.");
					break;
				}
				String resultTable = pageContent.substring(pageContent
						.indexOf("Elimination Results"), pageContent
						.indexOf("Swiss Rounds Results"));

				String regEx = "<td>(.+)</td><td>(.+)</td>";

				Pattern pattern = Pattern.compile(regEx);
				Matcher matcher = pattern.matcher(resultTable);

				while (matcher.find()) {
					int order = Integer.parseInt(matcher.group(1));
					String guildName = matcher.group(2).trim();

					for (Guild guild : guilds) {
						if (guild.getName().equals(guildName)) {
							guild.addOrder(order);
							break;
						}
					}
				}

				calTournament.add(Calendar.MONTH, 1);

			} catch (ClientProtocolException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

		}

	}

	public void voteForGvG(List<Guild> candidates)
			throws ClientProtocolException, IOException {

		if (candidates.size() != NUMBER_OF_VOTABLE_ITEMS) {
			logger.error("Unable to vote as " + candidates.size()
					+ " guild names were suppiled instead of " + NUMBER_OF_VOTABLE_ITEMS + ". Aborting voting for GvG.");
			return;
		}

		StringBuilder nodeIdsCSV = new StringBuilder();
		for (int i = 0; i < NUMBER_OF_VOTABLE_ITEMS; i++) {
			
			if (candidates.get(i) == null) {
				logger.error("Guild #" + (i+1) + " was not recognized. Aborting voting for GvG." );
				return;
			}
			
			nodeIdsCSV.append(candidates.get(i).getId());
			nodeIdsCSV.append(",");
		}

		logger.info("Voting For GvG: " + candidates.toString());

		HttpClient httpclient = new DefaultHttpClient();

		HttpPost httpost = new HttpPost(
				"https://www.guildwars.com/competitive/xunlai_tournament_house/login.php?lang=en");

		List<NameValuePair> nvps = new ArrayList<NameValuePair>();
		nvps
				.add(new BasicNameValuePair("hiddenNodeIds", nodeIdsCSV
						.toString()));
		nvps.add(new BasicNameValuePair("action", "insert"));
		nvps.add(new BasicNameValuePair("user_id", this.userId));
		nvps.add(new BasicNameValuePair("email_address", this.emailAddress));
		nvps.add(new BasicNameValuePair("password", this.password));
		nvps.add(new BasicNameValuePair("voteBallot", "GvG"));
		nvps.add(new BasicNameValuePair("continue", "Save Predictions"));
		nvps.add(new BasicNameValuePair("submit.x", "99"));
		nvps.add(new BasicNameValuePair("submit.y", "23"));
		nvps.add(new BasicNameValuePair("submit", "Submit"));

		httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));

		HttpResponse response = httpclient.execute(httpost);
		HttpEntity entity = response.getEntity();
		entity.consumeContent();
	}

	public List<Player> getQualifiedPlayers() {
		logger.info("Getting information about which players are qualified.");

		HttpPost httpost = new HttpPost(
				"https://www.guildwars.com/competitive/xunlai_tournament_house/login.php?lang=en");

		List<NameValuePair> nvps = new ArrayList<NameValuePair>();
		nvps.add(new BasicNameValuePair("continue", "View 1v1 Ballot"));
		nvps.add(new BasicNameValuePair("action", "insert"));
		nvps.add(new BasicNameValuePair("user_id", this.userId));
		nvps.add(new BasicNameValuePair("email_address", this.emailAddress));
		nvps.add(new BasicNameValuePair("password", this.password));

		List<Player> result = new LinkedList<Player>();

		try {
			httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));
			HttpResponse response = this.httpClient.execute(httpost);
			HttpEntity entity = response.getEntity();
			String pageContent = EntityUtils.toString(entity);

			String regEx = "class=\"dragableBox\" id=\"box(\\d+)\">([a-zA-Z ]+)<span";

			Pattern pattern = Pattern.compile(regEx);
			Matcher matcher = pattern.matcher(pageContent);

			while (matcher.find()) {
				Player player = new Player();
				player.setId(matcher.group(1));
				player.setName(matcher.group(2).trim());
				result.add(player);
			}

		} catch (UnsupportedEncodingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return result;
	}

	public void voteFor1v1(List<Player> candidates)
			throws ClientProtocolException, IOException {

		if (candidates.size() < NUMBER_OF_VOTABLE_ITEMS) {
			logger.error("Unable to vote as " + candidates.size()
					+ " player names were suppiled instead of " + NUMBER_OF_VOTABLE_ITEMS + ". Aborting voting for 1v1.");
			return;
		}

		StringBuilder nodeIdsCSV = new StringBuilder();
		for (int i = 0; i < NUMBER_OF_VOTABLE_ITEMS; i++) {
			
			if (candidates.get(i) == null) {
				logger.error("Player #" + (i+1) + " was not recognized. Aborting voting for 1v1." );
				return;
			}
			
			nodeIdsCSV.append(candidates.get(i).getId());
			nodeIdsCSV.append(",");
		}

		logger.info("Voting For 1v1: " + candidates.toString());

		HttpPost httpost = new HttpPost(
				"https://www.guildwars.com/competitive/xunlai_tournament_house/login.php?lang=en");

		List<NameValuePair> nvps = new ArrayList<NameValuePair>();
		nvps
				.add(new BasicNameValuePair("hiddenNodeIds", nodeIdsCSV
						.toString()));
		nvps.add(new BasicNameValuePair("action", "insert"));
		nvps.add(new BasicNameValuePair("user_id", this.userId));
		nvps.add(new BasicNameValuePair("email_address", this.emailAddress));
		nvps.add(new BasicNameValuePair("password", this.password));
		nvps.add(new BasicNameValuePair("vote_ballot", "1v1"));
		nvps.add(new BasicNameValuePair("continue", "Save Predictions"));
		nvps.add(new BasicNameValuePair("submit.x", "99"));
		nvps.add(new BasicNameValuePair("submit.y", "23"));
		nvps.add(new BasicNameValuePair("submit", "Submit"));

		httpost.setEntity(new UrlEncodedFormEntity(nvps, HTTP.UTF_8));

		HttpResponse response = this.httpClient.execute(httpost);
		HttpEntity entity = response.getEntity();
		entity.consumeContent();
	}

}
